////////////////////////////////////////////////////////////////////////////////////////
// VCO_MIDI
// Date: 13 Dec 2014
//
// MIDI control of AS10K through MCV Module: 
// - Added and LFO by software (only sine wave for now)
// - Added velocity note CV
// - Added calibration and pitch CV
//
// Using this module we can control up to 4 CV signals from 0 to 5 volts with 12 bits
// of resolution. 
//
// By Bleeps and Chips (J.Cano & Y. Torroja)
// www.bleepsandchips.com
//
////////////////////////////////////////////////////////////////////////////////////////
// CC BY-SA 3.0 - https://creativecommons.org/licenses/by-sa/3.0/
// Feel free to use and modify this code in any way you want
// Would be great if you maintain our authorship somehow :)
////////////////////////////////////////////////////////////////////////////////////////
#include <Arduino.h>

// Stop VCO on note off: Comment following line if you do not want VCO to stop
// when MIDI note_off is received (thats the normal mode of analog synthesisers, VCO
// is always running)
//#define STOP_VCO_ON_NOTE_OFF 1

// To simplify this development, this pins are fixed. Please, do not chage!
// GATE pin 
#define GATE_PIN 7   //-> To ENV Gate

#if defined(TCCR5B) && defined(TCCR5A) && defined(TIMSK5)  // For atmega2560/1280
// CS0 pins 
#define CS0_PIN  53  //-> To 4 MCV (CSO)
#define CS1_PIN  54  //-> To 4 MCV (CSO)
// Measure pin
#define FREQ_PIN 48  //
#else
// CS pins 
#define CS0_PIN  10  //-> To 4 MCV (CSO)
#define CS1_PIN  9   //-> To 4 MCV (CS1)
// Measure pin
#define FREQ_PIN 8   //
#endif

// Calibration pin
#define  CAL_PIN 12     // High level on this pin starts calibration
#define  DUMP_CAL_PIN 5 // High level on this pin dumps calibration data to serial port

//-----------------------------------------------------------------------------------------------------
// DAC modifications
//
// On Arduino UNO, Duecimila, Duemilanove...
// SCK -> PIN 13
// SDI -> PIN 11
// CS0 -> PIN 10
// CS1 -> PIN 9
//
// On Arduino MEGA...
// SCK -> PIN 52
// SDI -> PIN 51
// CS0 -> PIN 53
// CS1 -> PIN 54 // Not tested!
//
//-----------------------------------------------------------------------------------------------------

#define MIDI_NOTE_ON  0b10010000
#define MIDI_NOTE_OFF 0b10000000

////////////////////////////////////////////////////////////////////////////////////////
// LFO - Create vars and set them up
float        angle              = 0.0f;      //> Create a float type variable to be increased on every loop
float        DAC_value          = 0.0f;      //> Temporal variable to map sine of current angle to DAC values
unsigned int scaleFactor        = 2000;      //> Change this factor to obtain more or less resolution
unsigned int potValue           = 0;         //> Analog read (from 0 to 255)
float        potValueFloat      = 0.0f;      //> Analog read (float)
float        potValueFreqFactor = 25000.0f; //> Change this to modify LFO frecuency range
////////////////////////////////////////////////////////////////////////////////////////

void setup() {  
  // Initialize serial port to get midi messages
  Serial.begin(57600);

  // setup CV using DAC
  MCV_Setup(4096);

  // restore calibration
  loadCalibration();

  // setup calibration pin
  pinMode(CAL_PIN, INPUT);
  pinMode(GATE_PIN, OUTPUT);
}

void loop() {  

  /////////////////////////
  // Calibration section
  // Check calibration pin
  if (digitalRead(CAL_PIN)) {
    // Step by step calibration (more precise, max error about 1%, takes quite a long time to execute)
    unsigned long iniTime = millis();
    MCV_Calibration(10);
    Serial.print(F("Elapsed time (s): "));
    Serial.println((millis() - iniTime)/1000);

    saveCalibration();
  }

  /*
  /////////////////////////
   // Dump current calibration to serial port
   // Check calibration pin
   if (digitalRead(DUMP_CAL_PIN)) {
   dumpCalibrationData();
   }
   */

  updateLFO();

  /////////////////////////
  // MIDI playback section
  // We wait for incomming data
  if( Serial.available() != 0 )
  {

    byte readValue = Serial.read();
    byte note;
    byte vel;
    // We check more significant bit is 1 (MIDI command)
    if( (readValue & 0x80) != 0 )
    {

      // Esperamos dos bytes m\u00e1s...
      while( Serial.available() == 0 );
      note =  Serial.read();
      while( Serial.available() == 0 );
      vel  = Serial.read();  

      // Y tocamos la nota
      if( (readValue & 0b11110000) == MIDI_NOTE_ON ) // Check four most significant bits - NOTE ON
      {
        if (vel != 0)
        {
          MCV_SetNote(note,vel);
          MCV_Set_Gate(1);
        } 
        else {
#ifdef STOP_VCO_ON_NOTE_OFF
          MCV_Set(0);
#endif
          MCV_Set_Gate(0);
        }
      } 
      if( (readValue & 0b11110000) == MIDI_NOTE_OFF ) // Check four most significant bits - NOTE OFF
      {
#ifdef STOP_VCO_ON_NOTE_OFF
        MCV_Set(0);
#endif
        MCV_Set_Gate(0);
      }
    }
  }
}

/////////////////////////
// LFO: experimental
void updateLFO()
{
  // Increase the angle
  potValue = analogRead(A0);  
  potValueFloat = potValue/potValueFreqFactor;
  angle+=potValueFloat;
  //angle+=0.0001f;

  // As sine function goes from -1 to 1 (but our DAC expects unsigned integer values)
  // we add 1 to eliminate negative values
  DAC_value = sin( angle ) +1;

  // To increase resolution before mapping, lets multiply DAC_value by 
  // the scaleFactor (i.e 2000 to obtain a range from 0 to 4000) 
  DAC_value *= scaleFactor;

  // Then, map DAC_value to right DAC range, that is 0 to 4095 (or 0 to 5V)
  // Finally, send this value to channel A of MCV module
  MCV_Set( 'C', map( DAC_value, 0,scaleFactor*2,0,4095) );
}










